'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _command = require('@oclif/command');

var _base = require('../../base');

var _base2 = _interopRequireDefault(_base);

var _api = require('../../utils/api');

var _api2 = _interopRequireDefault(_api);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class GetCommand extends _base2.default {
	async run() {
		const { flags: { 'forging-status': showForgingStatus } } = this.parse(GetCommand);
		const client = (0, _api2.default)(this.userConfig.api);
		const baseInfo = await Promise.all([client.node.getConstants(), client.node.getStatus()]).then(([constantsResponse, statusResponse]) => Object.assign({}, constantsResponse.data, statusResponse.data));
		if (!showForgingStatus) {
			return this.print(baseInfo);
		}
		const fullInfo = await client.node.getForgingStatus().then(forgingResponse => Object.assign({}, baseInfo, {
			forgingStatus: forgingResponse.data || []
		})).catch(error => Object.assign({}, baseInfo, {
			forgingStatus: error.message
		}));
		return this.print(fullInfo);
	}
}

exports.default = GetCommand; /*
                               * LiskHQ/lisk-commander
                               * Copyright © 2017–2018 Lisk Foundation
                               *
                               * See the LICENSE file at the top-level directory of this distribution
                               * for licensing information.
                               *
                               * Unless otherwise agreed in a custom licensing agreement with the Lisk Foundation,
                               * no part of this software, including this file, may be copied, modified,
                               * propagated, or distributed except according to the terms contained in the
                               * LICENSE file.
                               *
                               * Removal or modification of this copyright notice is prohibited.
                               *
                               */

GetCommand.flags = Object.assign({}, _base2.default.flags, {
	'forging-status': _command.flags.boolean({
		description: 'Additionally provides information about forging status.'
	})
});

GetCommand.description = `
Gets information about a node.
`;

GetCommand.examples = ['node:get', 'node:get --forging-status'];