'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _base = require('../../base');

var _base2 = _interopRequireDefault(_base);

var _transactions = require('../../utils/transactions');

var _transactions2 = _interopRequireDefault(_transactions);

var _error = require('../../utils/error');

var _utils = require('../../utils/input/utils');

var _api = require('../../utils/api');

var _api2 = _interopRequireDefault(_api);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getTransactionInput = async () => {
	try {
		const { data } = await (0, _utils.getStdIn)({ dataIsRequired: true });
		if (!data) {
			throw new _error.ValidationError('No transaction was provided.');
		}
		return data;
	} catch (e) {
		throw new _error.ValidationError('No transaction was provided.');
	}
}; /*
    * LiskHQ/lisk-commander
    * Copyright © 2017–2018 Lisk Foundation
    *
    * See the LICENSE file at the top-level directory of this distribution
    * for licensing information.
    *
    * Unless otherwise agreed in a custom licensing agreement with the Lisk Foundation,
    * no part of this software, including this file, may be copied, modified,
    * propagated, or distributed except according to the terms contained in the
    * LICENSE file.
    *
    * Removal or modification of this copyright notice is prohibited.
    *
    */
class BroadcastCommand extends _base2.default {
	async run() {
		const { args: { transaction } } = this.parse(BroadcastCommand);
		const transactionInput = transaction || (await getTransactionInput());
		const transactionObject = (0, _transactions2.default)(transactionInput);
		const client = (0, _api2.default)(this.userConfig.api);
		const response = await client.transactions.broadcast(transactionObject);
		this.print(response.data);
	}
}

exports.default = BroadcastCommand;
BroadcastCommand.args = [{
	name: 'transaction',
	description: 'Transaction to broadcast in JSON format.'
}];

BroadcastCommand.flags = Object.assign({}, _base2.default.flags);

BroadcastCommand.description = `
Broadcasts a transaction to the network via the node specified in the current config.
Accepts a stringified JSON transaction as an argument, or a transaction can be piped from a previous command.
If piping make sure to quote out the entire command chain to avoid piping-related conflicts in your shell.
`;

BroadcastCommand.examples = ['broadcast transaction \'{"type":0,"amount":"100",...}\'', 'echo \'{"type":0,"amount":"100",...}\' | lisk transaction:broadcast'];